@@ -1,7 +1,7 @@
 /**
  * DinkC script engine
 
- * Copyright (C) 2008  Sylvain Beucler
+ * Copyright (C) 2008, 2009  Sylvain Beucler
 
  * This file is part of GNU FreeDink
 
@@ -49,11 +49,14 @@ struct global_function
 
 struct refinfo
 {
-  char name[10];
+  char* name;
   long location;
-  long current;
+  long current; // current offset
+  int cur_line; // current line
+  int cur_col;  // current column (position within ligne)
+  int debug_line; // last parsed line (whereas cur_line == next-to-be-read)
   int level;
-  long end;
+  long end; // size of the text, == strlen(rbuf[i])
   int sprite; //if more than 0, it was spawned and is owned by a sprite, if 1000 doesn't die
   /*bool*/int skipnext;
   int onlevel;
@@ -73,24 +76,20 @@ extern struct refinfo *rinfo[];
 
 extern void dinkc_init();
 extern void dinkc_quit();
-extern void decompress(FILE *in);
-extern void decompress_nocomp(FILE *in);
 extern int load_script(char filename[15], int sprite, /*bool*/int set_sprite);
 extern int dinkc_execute_one_liner(char* line);
 extern void strip_beginning_spaces(char *str);
-extern /*bool*/int locate(int script, char proc[20]);
-extern /*bool*/int locate_goto(char proc[50], int script);
-extern void decipher(char *crap, int script);
-extern void decipher_string(char line[200], int script);
+extern /*bool*/int locate(int script, char* proc_lookup);
+extern /*bool*/int locate_goto(char* expr, int script);
+extern long decipher(char* variable, int script);
+extern void decipher_string(char** line_p, int script);
 extern int add_callback(char name[20], int n1, int n2, int script);
 extern void kill_callback(int cb);
 extern void kill_callbacks_owned_by_script(int script);
 extern void kill_script(int k);
-extern void kill_all_scripts(void);
 extern void kill_all_scripts_for_real(void);
-extern /*bool*/int read_next_line(int script, char *line);
+extern char* read_next_line(int script);
 extern void process_callbacks(void);
-extern void init_scripts(void);
 extern int var_exists(char name[20], int scope);
 extern void make_int(char name[80], int value, int scope, int script);
 extern void var_equals(char name[20], char newname[20], char math, int script, char rest[200]);
@@ -98,11 +97,14 @@ extern int var_figure(char h[200], int script);
 extern void kill_scripts_owned_by(int sprite);
 extern void kill_returning_stuff(int script);
 extern void run_script(int script);
-extern void get_word(char line[300], int word, char *result);
 extern void attach(void);
 extern void int_prepare(char line[100], int script);
 extern void make_function(char file[10], char func[20]);
 
+/* Used by gfx_tiles.c only */
+extern void kill_all_scripts(void);
+extern void init_scripts(void);
+
 extern int returnint;
 extern int bKeepReturnInt; /* v1.08 */
 extern char returnstring[];
