@@ -1,7 +1,7 @@
 /**
  * Header for code common to FreeDink and FreeDinkedit
 
- * Copyright (C) 2005, 2007, 2008, 2009, 2010  Sylvain Beucler
+ * Copyright (C) 2005, 2007, 2008, 2009, 2010, 2012  Sylvain Beucler
 
  * This file is part of GNU FreeDink
 
@@ -51,7 +51,9 @@ struct attackinfo_struct
 
 struct sprite_placement
 {
-  int x, y, seq, frame, type, size;
+  int x, y;
+  int seq, frame, type;  /* DinkC: editor_seq, editor_frame, editor_type */
+  int size;
   BOOL_1BYTE active;
   int rotation, special, brain;
   
@@ -59,7 +61,7 @@ struct sprite_placement
   int speed, base_walk, base_idle, base_attack, base_hit, timer, que;
   int hard;
   rect alt; /* trim left/top/right/bottom */
-  int prop;
+  int is_warp;
   int warp_map;
   int warp_x;
   int warp_y;
@@ -220,6 +222,7 @@ extern void kill_text_owned_by(int sprite);
 
 /* Sprites - global */
 extern void kill_sprite_all (int sprite);
+extern int find_sprite(int editor_sprite);
 
 /* Scripts */
 extern int say_text_xy(char text[200], int mx, int my, int script);
