@@ -348,40 +348,3 @@ void write_lsb_short(short n, FILE *f)
   buf[1] = (n >> (1*8)) & 0xFF;
   fwrite(buf, 2, 1, f);
 }
-
-
-#ifdef TEST
-/* find ../.. -print0 | tr a-z A-Z | xargs -0 ./a.out */
-int
-main (int argc, char *argv[])
-{
-  if (argc == 1)
-    fprintf (stderr, "Usage: %s file...\n", argv[0]);
-  while (*++argv)
-    {
-      char *before = NULL;
-      char *after = NULL;
-      FILE *f = NULL;
-      before = strdup(*argv);
-      after = strdup(*argv);
-      ciconvert(after);
-      printf("%s => ", before);
-      printf("%s\n", after);
-      if ((strcasecmp(before, after) != 0)
-	  || ((f = fopen(after, "r")) == NULL))
-	fprintf(stderr, "ERROR with %s => %s\n", before, after);
-      if (f != NULL) fclose(f);
-      free(before);
-      free(after);
-    }
-  return 0;
-  /* other tests: */
-  /* - with multiple slashes: .//file */
-  /* - from the root: find /tmp/ ... */
-  /* - files containing '\' should fail */
-  /* - with a file and a directory with the same name, in the same
-       directory: currently we don't support that, as it's not
-       portable anyway */
-}
-
-#endif
