@@ -3,7 +3,7 @@
 
  * Copyright (C) 1997, 1998, 1999, 2002, 2003  Seth A. Robinson
  * Copyright (C) 2005, 2006  Dan Walma
- * Copyright (C) 2005, 2007, 2008, 2009, 2010, 2011  Sylvain Beucler
+ * Copyright (C) 2005, 2007, 2008, 2009, 2010, 2011, 2012  Sylvain Beucler
 
  * This file is part of GNU FreeDink
 
@@ -420,8 +420,8 @@ animate:
 				
 				if (spr[h].brain == 1) if (move_result > 100)
 				{
-					if (pam.sprite[move_result-100].prop == 1)
-						special_block(move_result - 100, h);
+					if (pam.sprite[move_result-100].is_warp == 1)
+						special_block(move_result - 100);
 				}
 				
 				
